--------------------------------------------------------------------------------
-- Copyright (c) 2001 by Trenz Electronic.
-- Duenner Kirchweg 77, 32257 Buende, Germany, www.trenz-electronic.de
--------------------------------------------------------------------------------
-- Project:     TE-XC2S Spartan-II Development System.
-- File:        teTop.vhd
-- Description: Generic FPGA top level with pad & timing constraints.
-- History:     2001nov13, FB, Created.
--------------------------------------------------------------------------------

library IEEE;   use IEEE.std_logic_1164.all;
library UNISIM; use UNISIM.vcomponents.all;

entity FPGA is
	port(
		clk_a1 : in STD_LOGIC;
		init   : in STD_LOGIC;
		ion_a5 : in STD_LOGIC;
		ion_b2 : in STD_LOGIC;
		iop_a5 : in STD_LOGIC;
		iop_b3 : in STD_LOGIC;
		ion_a1 : out STD_LOGIC;
		ion_a2 : out STD_LOGIC;
		ion_a3 : out STD_LOGIC;
		ion_a4 : out STD_LOGIC;
		ion_a6 : out STD_LOGIC;
		ion_b1 : out STD_LOGIC;
		ion_b3 : out STD_LOGIC;
		ion_b4 : out STD_LOGIC;
		ion_b5 : out STD_LOGIC;
		ion_b6 : out STD_LOGIC;
		iop_a1 : out STD_LOGIC;
		iop_a2 : out STD_LOGIC;
		iop_a3 : out STD_LOGIC;
		iop_a4 : out STD_LOGIC;
		iop_a6 : out STD_LOGIC;
		iop_b1 : out STD_LOGIC;
		iop_b2 : out STD_LOGIC;
		iop_b4 : out STD_LOGIC;
		iop_b5 : out STD_LOGIC;
		iop_b6 : out STD_LOGIC
		);
	
	-- pad locations
	attribute LOC: string;
	attribute LOC of clk_a1: signal is "P185";
	attribute LOC of iop_a1: signal is "P191";
	attribute LOC of ion_a1: signal is "P192";
	attribute LOC of iop_a2: signal is "P193";
	attribute LOC of ion_a2: signal is "P194";
	attribute LOC of iop_a3: signal is "P195";
	attribute LOC of ion_a3: signal is "P199";
	attribute LOC of iop_a4: signal is "P200";
	attribute LOC of ion_a4: signal is "P201";
	attribute LOC of iop_a5: signal is "P202";
	attribute LOC of ion_a5: signal is "P203";
	attribute LOC of iop_a6: signal is "P204";
	attribute LOC of ion_a6: signal is "P205";
	attribute LOC of iop_b1: signal is "P4";
	attribute LOC of ion_b1: signal is "P5";
	attribute LOC of iop_b2: signal is "P6";
	attribute LOC of ion_b2: signal is "P7";
	attribute LOC of iop_b3: signal is "P8";
	attribute LOC of ion_b3: signal is "P9";
	attribute LOC of iop_b4: signal is "P10";
	attribute LOC of ion_b4: signal is "P14";
	attribute LOC of iop_b5: signal is "P15";
	attribute LOC of ion_b5: signal is "P16";
	attribute LOC of iop_b6: signal is "P17";
	attribute LOC of ion_b6: signal is "P18";
	attribute LOC of init:   signal is "P107";
end FPGA;

architecture XILINX of FPGA is
	
	component tebl
		port (
			-- clocks
			CLK48  : out STD_LOGIC;
			CLK25  : out STD_LOGIC;
			-- push buttons, switches, leds, 7-segment
			PB     : out STD_LOGIC_VECTOR (7 downto 0);
			SW     : out STD_LOGIC_VECTOR (7 downto 0);
			LED    : in  STD_LOGIC_VECTOR (7 downto 0);
			DISP   : in  STD_LOGIC_VECTOR (15 downto 0);
			-- universal serial bus
			urxd   : out STD_LOGIC;
			urx0   : out STD_LOGIC;
			utxd   : in  STD_LOGIC;
			utx0   : in  STD_LOGIC;
			utxoe  : in  STD_LOGIC;
			-- vga
			VR     : in  STD_LOGIC;
			VG     : in  STD_LOGIC;
			VB     : in  STD_LOGIC;
			VI     : in  STD_LOGIC;
			VROW   : out STD_LOGIC_VECTOR (9 downto 0);
			VCOL   : out STD_LOGIC_VECTOR (9 downto 0);
			VHBLK  : out STD_LOGIC;
			VVBLK  : out STD_LOGIC;
			-- expansion connector pins
			clk_a1 : in  STD_LOGIC;
			iop_a1 : out STD_LOGIC;
			ion_a1 : out STD_LOGIC;
			iop_a2 : out STD_LOGIC;
			ion_a2 : out STD_LOGIC;
			iop_a3 : out STD_LOGIC;
			ion_a3 : out STD_LOGIC;
			iop_a4 : out STD_LOGIC;
			ion_a4 : out STD_LOGIC;
			iop_a6 : out STD_LOGIC;
			ion_a6 : out STD_LOGIC;
			iop_a5 : in  STD_LOGIC;
			ion_a5 : in  STD_LOGIC;
			iop_b1 : out STD_LOGIC;
			ion_b1 : out STD_LOGIC;
			iop_b2 : out STD_LOGIC;
			ion_b2 : in  STD_LOGIC;
			iop_b3 : in  STD_LOGIC;
			ion_b3 : out STD_LOGIC;
			iop_b4 : out STD_LOGIC;
			ion_b4 : out STD_LOGIC;
			iop_b5 : out STD_LOGIC;
			ion_b5 : out STD_LOGIC;
			iop_b6 : out STD_LOGIC;
			ion_b6 : out STD_LOGIC
			);
	end component;
	
	component Core
		port (
			clk48 : in  STD_LOGIC;
			clk25 : in  STD_LOGIC;
			rst   : in  STD_LOGIC;
			-- push buttons, switches, leds, 7-segment
			pb    : in  STD_LOGIC_VECTOR (7 downto 0);
			sw    : in  STD_LOGIC_VECTOR (7 downto 0);
			disp  : out STD_LOGIC_VECTOR (15 downto 0);
			led   : out STD_LOGIC_VECTOR (7 downto 0);
			-- usb
			rxd   : in  STD_LOGIC;
			rx0   : in  STD_LOGIC;
			txd   : out STD_LOGIC;
			tx0   : out STD_LOGIC;
			txoe  : out STD_LOGIC;
			-- vga
			r     : out STD_LOGIC;
			g     : out STD_LOGIC;
			b     : out STD_LOGIC;
			i     : out STD_LOGIC;
			row   : in  STD_LOGIC_VECTOR (9 downto 0);
			col   : in  STD_LOGIC_VECTOR (9 downto 0);
			hblk  : in  STD_LOGIC;
			vblk  : in  STD_LOGIC
			);
	end component;
	
	-- clocks
	signal CLK48    : STD_LOGIC;
	signal CLK25    : STD_LOGIC;
	-- push buttons, switches, leds, 7-segment
	signal PB       : STD_LOGIC_VECTOR (7 downto 0);
	signal SW       : STD_LOGIC_VECTOR (7 downto 0);
	signal LED      : STD_LOGIC_VECTOR (7 downto 0);
	signal DISP     : STD_LOGIC_VECTOR (15 downto 0);
	-- universal serial bus
	signal urxd     : STD_LOGIC;
	signal urx0     : STD_LOGIC;
	signal utxd     : STD_LOGIC;
	signal utx0     : STD_LOGIC;
	signal utxoe    : STD_LOGIC;
	-- vga
	signal VR       : STD_LOGIC;
	signal VG       : STD_LOGIC;
	signal VB       : STD_LOGIC;
	signal VI       : STD_LOGIC;
	signal VROW     : STD_LOGIC_VECTOR (9 downto 0);
	signal VCOL     : STD_LOGIC_VECTOR (9 downto 0);
	signal VHBLK    : STD_LOGIC;
	signal VVBLK    : STD_LOGIC;
	-- reset & buffered clocks
	signal rst_i    : STD_LOGIC;
	signal clk25_i  : STD_LOGIC;
	signal clk_a1_i : STD_LOGIC;
	
	-- timing constraints
	attribute PERIOD: string;
	attribute PERIOD of clk_a1: signal is "48MHz";
	attribute PERIOD of clk25:  signal is "24MHz";
	
begin
	Ubuf48 : BUFG port map (
		I=> clk_a1,
		O=> clk_a1_i);
	
	Ubuf25 : BUFG port map (
		I=> clk25,
		O=> clk25_i);
	
	rst_i<= not(init);
	
	Utebl : tebl port map (
		-- clocks
		CLK48  => clk48,
		CLK25  => clk25,
		-- push buttons, switches, leds, 7-segment
		PB     => pb,
		SW     => sw,
		LED    => led,
		DISP   => disp,
		-- universal serial bus
		urxd   => urxd,
		urx0   => urx0,
		utxd   => utxd,
		utx0   => utx0,
		utxoe  => utxoe,
		-- vga
		VR     => vr,
		VG     => vg,
		VB     => vb,
		VI     => vi,
		VROW   => vrow,
		VCOL   => vcol,
		VHBLK  => vhblk,
		VVBLK  => vvblk,
		-- expansion connector pins
		clk_a1 => clk_a1_i,
		iop_a1 => iop_a1,
		ion_a1 => ion_a1,
		iop_a2 => iop_a2,
		ion_a2 => ion_a2,
		iop_a3 => iop_a3,
		ion_a3 => ion_a3,
		iop_a4 => iop_a4,
		ion_a4 => ion_a4,
		iop_a6 => iop_a6,
		ion_a6 => ion_a6,
		iop_a5 => iop_a5,
		ion_a5 => ion_a5,
		iop_b1 => iop_b1,
		ion_b1 => ion_b1,
		iop_b2 => iop_b2,
		ion_b2 => ion_b2,
		iop_b3 => iop_b3,
		ion_b3 => ion_b3,
		iop_b4 => iop_b4,
		ion_b4 => ion_b4,
		iop_b5 => iop_b5,
		ion_b5 => ion_b5,
		iop_b6 => iop_b6,
		ion_b6 => ion_b6);
	
	Ucore : Core port map (
		clk48 => clk48,
		clk25 => clk25_i,
		rst   => rst_i,
		-- push buttons, switches, leds, 7-segment
		pb    => pb,
		sw    => sw,
		disp  => disp,
		led   => led,
		-- usb
		rxd   => urxd,
		rx0   => urx0,
		txd   => utxd,
		tx0   => utx0,
		txoe  => utxoe,
		-- vga
		r     => vr,
		g     => vg,
		b     => vb,
		i     => vi,
		row   => vrow,
		col   => vcol,
		hblk  => vhblk,
		vblk  => vvblk);
	
end XILINX;

--------------------------------------------------------------------------------
-- end of file
